#if defined _enginesound_included
 #endinput
#endif
#define _enginesound_included

#include <core>

//-----------------------------------------------------------------------------
// common volume values
//-----------------------------------------------------------------------------
#define VOL_NORM		1.0

//-----------------------------------------------------------------------------
// common attenuation values
//-----------------------------------------------------------------------------
#define ATTN_NONE		0.0
#define ATTN_NORM		0.8
#define ATTN_IDLE		2.0
#define ATTN_STATIC		1.25
#define ATTN_RICOCHET	1.5
#define ATTN_GUNFIRE	0.27

#define MAX_ATTENUATION		3.98

//-----------------------------------------------------------------------------
// common pitch values
//-----------------------------------------------------------------------------
#define	PITCH_NORM		100			// non-pitch shifted
#define PITCH_LOW		95			// other values are possible - 0-255, where 255 is very high
#define PITCH_HIGH		120

//-----------------------------------------------------------------------------
// channels
//-----------------------------------------------------------------------------
enum
{
	CHAN_REPLACE	= -1,
	CHAN_AUTO		= 0,
	CHAN_WEAPON		= 1,
	CHAN_VOICE		= 2,
	CHAN_ITEM		= 3,
	CHAN_BODY		= 4,
	CHAN_STREAM		= 5,		// allocate stream channel from the static or dynamic area
	CHAN_STATIC		= 6,		// allocate channel from the static area 
	CHAN_VOICE_BASE	= 7,		// allocate channel for network voice data
	CHAN_USER_BASE	= (CHAN_VOICE_BASE+128)		// Anything >= this number is allocated to game code.
};

//-----------------------------------------------------------------------------
// soundlevels
//-----------------------------------------------------------------------------
enum soundlevel_t
{
	SNDLVL_NONE			= 0,
	SNDLVL_20dB			= 20,			// rustling leaves
	SNDLVL_25dB			= 25,			// whispering
	SNDLVL_30dB			= 30,			// library
	SNDLVL_35dB			= 35,
	SNDLVL_40dB			= 40,
	SNDLVL_45dB			= 45,			// refrigerator
	SNDLVL_50dB			= 50,	// 3.9	// average home
	SNDLVL_55dB			= 55,	// 3.0
	SNDLVL_IDLE			= 60,	// 2.0	
	SNDLVL_60dB			= 60,	// 2.0	// normal conversation, clothes dryer
	SNDLVL_65dB			= 65,	// 1.5	// washing machine, dishwasher
	SNDLVL_STATIC		= 66,	// 1.25
	SNDLVL_70dB			= 70,	// 1.0	// car, vacuum cleaner, mixer, electric sewing machine
	SNDLVL_NORM			= 75,
	SNDLVL_75dB			= 75,	// 0.8	// busy traffic
	SNDLVL_80dB			= 80,	// 0.7	// mini-bike, alarm clock, noisy restaurant, office tabulator, outboard motor, passing snowmobile
	SNDLVL_TALKING		= 80,	// 0.7
	SNDLVL_85dB			= 85,	// 0.6	// average factory, electric shaver
	SNDLVL_90dB			= 90,	// 0.5	// screaming child, passing motorcycle, convertible ride on frw
	SNDLVL_95dB			= 95,
	SNDLVL_100dB		= 100,	// 0.4	// subway train, diesel truck, woodworking shop, pneumatic drill, boiler shop, jackhammer
	SNDLVL_105dB		= 105,			// helicopter, power mower
	SNDLVL_110dB		= 110,			// snowmobile drvrs seat, inboard motorboat, sandblasting
	SNDLVL_120dB		= 120,			// auto horn, propeller aircraft
	SNDLVL_130dB		= 130,			// air raid siren
	SNDLVL_GUNFIRE		= 140,	// 0.27	// THRESHOLD OF PAIN, gunshot, jet engine
	SNDLVL_140dB		= 140,	// 0.2
	SNDLVL_150dB		= 150,	// 0.2
	SNDLVL_180dB		= 180,			// rocket launching
	// NOTE: Valid soundlevel_t values are 0-255.
};

//-----------------------------------------------------------------------------
// soundflags
//-----------------------------------------------------------------------------
enum SoundFlags_t
{
	SND_NOFLAGS			= 0,			// to keep the compiler happy
	SND_CHANGE_VOL		= (1<<0),		// change sound vol
	SND_CHANGE_PITCH	= (1<<1),		// change sound pitch
	SND_STOP			= (1<<2),		// stop the sound
	SND_SPAWNING		= (1<<3),		// we're spawning, used in some cases for ambients
										// not sent over net, only a param between dll and server.
	SND_DELAY			= (1<<4),		// sound has an initial delay
	SND_STOP_LOOPING	= (1<<5),		// stop all looping sounds on the entity.
	SND_SPEAKER			= (1<<6),		// being played again by a microphone through a speaker
	SND_SHOULDPAUSE		= (1<<7),		// this sound should be paused if the game is paused
};

//-----------------------------------------------------------------------------
// natives
//-----------------------------------------------------------------------------

/**
 * Prefetchs a given sound.
 *
 * @param sample		Name of the sound.
 * @noreturn
 */
native PrefetchSound(const String:sample[]);

/**
 * Gets the duration of a given sound.
 *
 * @param sample		Name of the sound.
 * @return				The duration.
 */
native Float:GetSoundDuration(const String:sample[]);

/**
 * Emit a given sound to one/all client(s).
 *
 * @param client			Index of the client to 'emit' the sound to (0 = all).
 * @param entity			Index of the entity to 'emit' the sound from.
 * @param channel			Channel of the sound.
 * @param sample			Name of the sound.
 * @param volume			Volume of the sound.
 * @param attenuation		Attenuation of the sound.
 * @param flags				Flags of the sound.
 * @param pitch				Pitch of the sound.
 * @param origin			Origin of the sound ({0.0,0.0,0.0} = ignore)
 * @param direction			Direction of the sound ({0.0,0.0,0.0} = ignore)
 * @param updateposition	Update the position of the sound (e.g. when the sound is attached to a player and he moves)
 * @param soundtime			How long the sound should play.
 * @param speakerentity
 * @noreturn
 */
native EmitSound(client, entity, channel, const String:sample[], Float:volume, Float:attenuation, flags=0, pitch=PITCH_NORM, Float:origin[3]={0.0,0.0,0.0}, Float:direction[3]={0.0,0.0,0.0}, bool:updateposition=true, Float:soundtime=0.0, speakerentity=-1);

/**
 * Emit a given sound to one/all client(s).
 * Same as EmitSound but it uses the soundlevel instead of attenuation.
 *
 * @param client			Index of the client to 'emit' the sound to (0 = all).
 * @param entity			Index of the entity to 'emit' the sound from.
 * @param channel			Channel of the sound.
 * @param sample			Name of the sound.
 * @param volume			Volume of the sound.
 * @param level				Soundlevel of the sound.
 * @param flags				Flags of the sound.
 * @param pitch				Pitch of the sound.
 * @param origin			Origin of the sound ({0.0,0.0,0.0} = ignore)
 * @param direction			Direction of the sound ({0.0,0.0,0.0} = ignore)
 * @param updateposition	Update the position of the sound (e.g. when the sound is attached to a player and he moves)
 * @param soundtime			How long the sound should play.
 * @param speakerentity
 * @noreturn
 */
native EmitSoundL(client, entity, channel, const String:sample[], Float:volume, soundlevel_t:level, flags=0, pitch=PITCH_NORM, Float:origin[3]={0.0,0.0,0.0}, Float:direction[3]={0.0,0.0,0.0}, bool:updateposition=true, Float:soundtime=0.0, speakerentity=-1);

/**
 * I don't know ^^
 * "Emit a sentence and also a close caption token for the sentence as appropriate."
 *
 * @param client			Index of the client to 'emit' the sound to (0 = all).
 * @param entity			Index of the entity to 'emit' the sound from.
 * @param channel			Channel of the sound.
 * @param sentence			Sentence of the sound.
 * @param volume			Volume of the sound.
 * @param level				Soundlevel of the sound.
 * @param flags				Flags of the sound.
 * @param pitch				Pitch of the sound.
 * @param origin			Origin of the sound ({0.0,0.0,0.0} = ignore)
 * @param direction			Direction of the sound ({0.0,0.0,0.0} = ignore)
 * @param updateposition	Update the position of the sound (e.g. when the sound is attached to a player and he moves)
 * @param soundtime			How long the sound should play.
 * @param speakerentity
 * @noreturn
 */
native EmitSentenceByIndex(client, entity, channel, sentence, Float:volume, soundlevel_t:level, flags=0, pitch=PITCH_NORM, Float:origin[3]={0.0,0.0,0.0}, Float:direction[3]={0.0,0.0,0.0}, bool:updateposition=true, Float:soundtime=0.0, speakerentity=-1);

/**
 * Stops an 'emited' sound of a given entity.
 *
 * @param entity			Index of the entity.
 * @param channel			Channel of the sound.
 * @param sample			Name of the sound.
 * @noreturn
 */
native StopSound(entity, channel, const String:sample[]);

/**
 * Emit an 'ambient' sound that isn't spatialized.
 *
 * @param sample			Name of the sound.
 * @param volume			Volume of the sound.
 * @param pitch				Pitch of the sound.
 * @param flags				Flags of the sound.
 * @param soundtime			How long the sound should play.
 * @noreturn
 */
native EmitAmbientSound(const String:sample[], Float:volume, pitch=PITCH_NORM, flags=0, Float:soundtime=0.0);

/**
 * Do not edit below this line!
 */
public Extension:__ext_enginesound =
{
	name = "EngineSound Extension",
	file = "enginesound.ext",
#if defined AUTOLOAD_EXTENSIONS
	autoload = 1,
#else
	autoload = 0,
#endif
#if defined REQUIRE_EXTENSIONS
	required = 1,
#else
	required = 0,
#endif
};